/**
 * \file gre/autolabelcontext.h
 * \brief GRE::AUTOLABELCONTEXT definitions
 *
 * \if NODOC
 * $Id: autolabelcontext.h_v 1.1 2012/05/22 17:53:25 mju Exp $
 *
 * $Log: autolabelcontext.h_v $
 * Revision 1.1  2012/05/22 17:53:25  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_AUTOLABELCONTEXT_H
#define	INC_GRE_AUTOLABELCONTEXT_H

#ifndef	INC_GRE_SCRIPTCONTEXT_H
#include <gre/scriptcontext.h>
#endif

#ifndef	INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class GEOSPATIAL;
	}
#endif


namespace GRE {

//! Context for getting autolabel data values.
class AUTOLABELCONTEXT {
	public:

		enum FLAGS {
			FLAGS_None =				0x00,
			FLAG_AllowOptimize =		0x01,
			};

		//! Constructor.
		AUTOLABELCONTEXT ();

		//! Destructor.
		~AUTOLABELCONTEXT ();

		//! Determine if has been setup.
		bool IsSetup () const
			{ return (m_bIsSetup); }

		// Get values.
		// Return TRUE if have string, FALSE if don't, < 0 if error.
		int GetValues (
			ELEMNUM ElemNum,							//!< Element number
			MISTRING& labelstr,						//!< Label text string returned
			double *pPriority = 0,					//!< Label priority returned if desired
			LABELFRAMEPARMS *pFrameParms = 0		//!< Frame parameters passed/returned if desired
			);

		//! Setup if parameters set to do labeling.
		//! Use AUTOLABELPARMS::CheckNeedDb to determine if database object is needed.
		ERRVALUE Setup (
			GRE::LAYERDC& layerdc,					//!< Layer drawing context
			const AUTOLABELPARMS& LabelParm,		//!< Label parameters
			RVC::GEOMETRIC& ObjGeometric,			//!< Open geometric object
			ELEMTYPE ElemType,						//!< Element type
			FLAGS flags = FLAGS_None				//!< Flags
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RVC::DBTABLE m_TableText;
		RVC::DBTABLE m_TablePriority;
		RVC::DBTABLE::RECORD m_RecordText;
		RVC::DBTABLE::RECORD m_RecordPriority;
		GRE::SCRIPTCONTEXT_LABEL m_LabelSC;
		SIMPLE_ARRAY<INT32> m_RecNums;
		int m_FieldNumText;
		int m_FieldNumPriority;
		int m_QuerySeg;
		ELEMTYPE m_ElemType;
		bool m_bIsSetup;
		UINT8 m_TextMode;
		UINT8 m_PriorityMode;

		void Release ();

		AUTOLABELCONTEXT (const AUTOLABELCONTEXT& rhs);
		AUTOLABELCONTEXT& operator= (const AUTOLABELCONTEXT& rhs);
		#endif
	};

}	// End namespace GRE

#endif	// INC_GRE_AUTOLABELCONTEXT_H
