/**
 * \file gre/lrtin.h
 * \brief GRE_LAYER_TIN class definition.
 *
 * \if NODOC
 * $Id: lrtin.h_v 1.37 2010/08/19 17:40:45 mju Exp $
 *
 * $Log: lrtin.h_v $
 * Revision 1.37  2010/08/19 17:40:45  mju
 * Use elemnum type.
 *
 * Revision 1.36  2010/04/23 16:54:55  mju
 * Use unsignedset.
 *
 * Revision 1.35  2008/05/15 17:55:14  mju
 * Chg v_constructLayerSEction.
 *
 * Revision 1.34  2008/05/12 16:59:57  mju
 * Add parm to constructLayerSection.
 *
 * Revision 1.33  2008/03/17 21:33:47  mju
 * Change v_getLayerView to v_getLayerSection.
 *
 * Revision 1.32  2007/03/12 17:42:56  mju
 * Add v_setObjectDescriptor.
 *
 * Revision 1.31  2006/10/10 13:59:12  mju
 * Use miconstruct* instead of micon.
 *
 * Revision 1.30  2006/10/09 17:51:31  mju
 * Rmv v_setDftName.
 *
 * Revision 1.29  2006/10/02 17:48:20  mju
 * DataTip parms return as pointer.
 *
 * Revision 1.28  2006/09/29 14:45:04  scowan
 * Set objitem to its namespace.
 *
 * Revision 1.27  2006/09/28 13:14:32  mju
 * Use elemparms hdr.
 *
 * Revision 1.26  2006/09/15 15:31:10  mju
 * Serializer changes.
 *
 * Revision 1.25  2006/01/05 17:50:26  dwilliss
 * changed parm type on a method
 *
 * Revision 1.24  2005/11/28 16:38:32  mju
 * Treeview update legend.
 *
 * Revision 1.23  2005/11/04 18:52:50  mju
 * enums in gre namespace.
 *
 * Revision 1.22  2005/10/26 16:35:53  mju
 * Use gre namespace instead of legacy types.
 *
 * Revision 1.21  2005/10/14 17:32:18  mju
 * GRE namespace and treeview.
 *
 * Revision 1.20  2005/03/07 17:24:39  scowan
 * Fixed g++ warings.
 *
 * Revision 1.19  2005/03/01 14:22:25  mju
 * *** empty log message ***
 *
 * Revision 1.18  2005/02/25 23:22:48  scowan
 * Added copy virtual method implementation.
 *
 * Revision 1.17  2005/01/28 17:17:15  dwilliss
 * Fixed doxygen comment in header
 *
 * Revision 1.16  2005/01/21 14:53:54  mju
 * Forward declare rvc::tin class.
 *
 * Revision 1.15  2005/01/20 16:57:54  mju
 * Redesign layer type registration.
 *
 * Revision 1.14  2004/10/01 17:20:20  mju
 * Add dlgparent to assignObject.
 *
 * Revision 1.13  2004/07/12 23:14:17  mju
 * No georegion.
 *
 * Revision 1.12  2004/03/05 17:51:30  mju
 * Make dtor virtual if have virtual fns.
 *
 * Revision 1.11  2004/02/13 15:49:15  scowan
 * Fixed deprecated mgd items.
 *
 * Revision 1.10  2004/02/06 16:01:54  mju
 * Use GRE::FINDFLAGS.
 *
 * Revision 1.9  2003/10/15 17:09:12  scowan
 * Removed get object filter forward declaration.
 *
 * Revision 1.8  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.7  2003/07/30 15:30:04  mju
 * Ignore private sections.
 *
 * Revision 1.6  2002/10/23 22:07:21  mju
 * Add v_GetElemSetByExtent.
 *
 * Revision 1.5  2002/09/23 15:23:31  mju
 * Add openprivateobject().
 *
 * Revision 1.4  2002/09/20 17:03:07  mju
 * Only declare v_ControlPanel if X or MFC.
 *
 * Revision 1.3  2002/09/19 17:08:55  mju
 * Make tin obj a pointer so no dependency on tin header.
 *
 * Revision 1.2  2002/09/09 21:13:42  mju
 * Fix inclusion guard.
 *
 * Revision 1.1  2002/09/09 20:37:35  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRTIN_H
#define	INC_GRE_LRTIN_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_GRE_ELEMPARMS_H
#include <gre/elemparms.h>
#endif

#ifndef	INC_GRE_LEGEND_H
#include <gre/legend.h>
#endif

namespace RVC {
	class TIN;
	}

namespace GRE {
//===================================================================================================================
//! GRE layer class for viewing TIN (Triangulated Irregular Network) spatial object.
class LAYER_TIN : public LAYER {
	public:

		// LAYER_TIN::DISPPARM
		class DISPPARM : public LAYER::DISPPARM {
			public:

				class DLG;										//!< Layer settings dialog, defined in <gre/uitin.h>

				RVC::OBJITEM m_ObjItem;
				RVC::OBJECTNAME m_GeorefName;				//!< Name of georeference object
				RVC::OBJITEM m_ObjItemStyle;
				POINTPARMS Node;
				LINEPARMS Edge;
				POLYPARMS Triangle;
				struct CONTOUR {
					UINT8 Selected;
					LINESTYLE Style;
					double Interval;
					} Contour[2];
				UINT8 ContourUnitCode;
				bool m_IsEditable;
				bool m_HasSaveInfo;
				mutable RVC::OBJITEM m_SaveObjItem;
				mutable RVC::OBJITEM m_SaveObjItemStyle;

				DISPPARM ();
				DISPPARM (
					const DISPPARM& rhs
					): LAYER::DISPPARM(rhs) { Copy(rhs); }
				virtual ~DISPPARM (
					) { Free(); }

				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Free(); Copy(rhs); } return (*this); }

				//! Set to use specified object.
				ERRVALUE SetObject (
					const RVC::OBJITEM& objitem,		//!< Object to set
					bool keepsettings = false		//!< Retain current display settings
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				void Copy (const DISPPARM&);
				void Free ();
				static ERRVALUE SerializerCB_StyleFileName (SERIALIZER&, const SERIALIZERITEM *, void *, SERIALIZERITEM::ACTION);
				void UpdateSaveInfo () const;

				//! Overrides from SERIALIZABLE.
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

				//! Overrides from LAYER::DISPPARM.
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE vSetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE Validate ();

				friend class LAYER_TIN;
				#endif //!< GENERATING_DOXYGEN_OUTPUT
			};	//! End of LAYER_TIN::DISPPARM

		//! Register this layer type
		static void RegisterType (
			);

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT *context
			);

		//! Construct layer and insert into list
		explicit LAYER_TIN (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_TIN ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	public:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIVDRAWDATA;
		#endif //!< GENERATING_DOXYGEN_OUTPUT

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::TIN *m_pObjTIN;
		GRE::LEGEND_POINT m_NodeLegend;
		GRE::LEGEND_LINE m_EdgeLegend;
		GRE::LEGEND_POLYGON m_TriangleLegend;
		GRE::LEGEND_LINE m_ContourLegend;
		COMPINFO m_NodeCompInfo;
		COMPINFO m_EdgeCompInfo;
		COMPINFO m_TriangleCompInfo;

		static void LegendMsgHandler (GRAPHICTREEITEM_MSG *msg);

		ERRVALUE OpenPrivateObject ();
		void SetupLegend ();
		int SetupRegion ();

		//! Overrides from LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
		virtual bool v_CanShowDetails () const;
		virtual GRE::LAYERSECTION* v_ConstructLayerSection (GRE::VIEW* view);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0);
	#endif
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual const DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual ERRVALUE v_DataTipSetParms (int component, const DATATIPPARMS& DataTipParms);
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual ERRVALUE v_DrawElementLow (LAYERDC& layerdc, ELEMTYPE ElemType, ELEMNUM ElemNum, const void *style);
		virtual ERRVALUE v_DrawElementsLow (LAYERDC& layerdc, ELEMTYPE ElemType, const UNSIGNEDSET *pElemSet, const void *style);
		virtual ELEMNUM v_FindClosestElement (const DPOINT2D& point, double searchdist, double *distret, int component, const UNSIGNEDSET *pSearchSet, GRE::FINDFLAGS findflags);
		virtual int v_GetComponent (ELEMTYPE ElemType) const;
		virtual MICONSTRUCT* v_GetComponentIcon (int component) const;
		virtual COMPINFO* v_GetComponentInfo (int component);
		virtual MISTRING v_GetComponentName (int component) const;
		virtual const void* v_GetElemDefaultStyle (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawSelectionMode (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawStyleMode (ELEMTYPE ElemType) const;
		virtual ERRVALUE v_GetElemExtents (ELEMTYPE ElemType, ELEMNUM ElemNum, DRECT3D& Extents);
		virtual ERRVALUE v_GetElemSetByExtent (ELEMTYPE ElemType, const DRECT2D& Extent, UNSIGNEDSET& ElemSet);
		virtual ELEMTYPE v_GetElemType (int component) const;
		virtual bool v_GetGeorefName (RVC::OBJECTNAME& name, int component = 0) const;
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual INT64 v_GetNumElements (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual ERRVALUE v_GetStyleInfo (ELEMTYPE ElemType, RVC::OBJITEM& ObjItem, RVC::OBJECTNAME& TableName);
		virtual bool v_HasLegend () const;
		virtual bool v_IsTableUsableForComponent (int component, const RVC::DBTABLE& table) const;
	#ifdef USE_MGUI
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual ERRVALUE v_ReportRawElemInfo (MISTRING& infostr, ELEMTYPE ElemType, ELEMNUM ElemNum);
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetElemDrawSelectionMode (ELEMTYPE ElemType, UINT16 SelectMode);
		virtual void v_SetElemDrawStyleMode (ELEMTYPE ElemType, UINT16 StyleMode);
		virtual void v_SetLegendSampleFlags (MGD::STYLESAMPLEFLAGS SampleFlags);
		virtual void v_SetObjectDescriptor (int component, const RVC::DESCRIPTOR& descriptor);
		virtual ERRVALUE v_SetStyleObjItem (const RVC::OBJITEM& objitem);
		virtual int v_TestElementByRegion (ELEMTYPE ElemType, ELEMNUM ElemNum, const REGION2D& Region, TESTCRITERIA TestMode);
	#ifdef USE_MGUI
		virtual MGUI::CTRL_TREE::ITEMHND v_TreeViewAddUpdateLegend (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
		virtual void v_TreeViewAddUpdateDetails (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
	#endif
		virtual int v_UpdateExtents (bool forceupdate);

		friend class PRIVDRAWDATA;
		
		LAYER_TIN (const LAYER_TIN&);
		LAYER_TIN& operator= (const LAYER_TIN&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
}	// End namespace GRE


#endif	//!< INC_GRE_LRTIN_H
