/**
 * \file mgui/dlgselectradio.h
 * \brief MGUI::DLG_SELECT_RADIO class definition
 *
 * \if NODOC
 * $Id: dlgselectradio.h_v 1.2 2011/08/26 13:32:06 mju Exp $
 *
 * $Log: dlgselectradio.h_v $
 * Revision 1.2  2011/08/26 13:32:06  mju
 * Use formradiobuttons hdr.
 *
 * Revision 1.1  2010/01/28 17:30:44  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGSELECTRADIO_H
#define	INC_MGUI_DLGSELECTRADIO_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_FORMRADIOBUTTONS_H
#include <mgui/formradiobuttons.h>
#endif

#ifndef	INC_MGUI_LISTITEM_H
#include <mgui/listitem.h>
#endif

namespace MGUI {

//! Dialog to display a set of radiobutton choices to select from.
class DLG_SELECT_RADIO : public DLGSHELL {
	public:

		//! Construct with title.
		explicit DLG_SELECT_RADIO (
			const MISTRING& dlgtitle,				//!< Text for title
			HELPID helpid = HELPID__None			//!< Help ID
			);

		//! Constructor with title and prompt string.
		DLG_SELECT_RADIO (
			const MISTRING& dlgtitle,				//!< Text for title
			const MISTRING& prompt,					//!< Text for prompt
			HELPID helpid = HELPID__None			//!< Help ID
			);

		//! Destructor.
		~DLG_SELECT_RADIO (
			);

		//! Add choice, must be used before the call to DoModal().
		void AddChoice (
			MGUI::ID id,						//!< MGUI::ID to assign to the item
			const MISTRING& string			//!< The string to show
			);
			
		//! Get the selected choice ID.
		//! If the dialog is created using FLAG_AllowNone and the user selects \<None\>, the return value will be EUserCancel
		//! @return MGUI::ID of the selected choice.
		MGUI::ID GetSelectedChoiceID (
			) { return (m_Rbn.GetSelectedID()); }

		//! Set the default choice using the MGUI::ID given in AddChoice().
		void SetDefaultChoiceID (
			MGUI::ID id
			) { m_DefaultID = id; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MILIST<MGUI::LISTITEM> m_ChoiceList;
		CTRL_LABEL m_PromptLabel;
		FORM_RADIOBUTTONS m_Rbn;
		MGUI::ID m_DefaultID;
		MISTRING m_PromptString;

		virtual ERRVALUE v_CreateContent ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace MGUI

#endif	// INC_MGUI_DLGSELECTRADIO_H
