/**
 * \file mgui/formsnapparms.h
 * \brief RVC::VECTORTOOLKIT::SNAPPARMS::PANEL class definitions.
 *
 * \if NODOC
 * $Id: formsnapparms.h_v 1.5 2011/09/01 19:34:05 mju Exp $
 *
 * $Log: formsnapparms.h_v $
 * Revision 1.5  2011/09/01 19:34:05  mju
 * dox hdr.
 *
 * Revision 1.4  2006/04/13 16:23:02  dwilliss
 * doxygen
 *
 * Revision 1.3  2005/06/24 21:01:52  mju
 * Don't use mgui templates.
 * Add ability to set delegates, make overridables protected and document when to call.
 *
 * Revision 1.2  2005/06/23 21:34:02  mju
 * Exclude private section from doxygen.
 *
 * Revision 1.1  2004/02/04 21:43:56  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMSNAPPARMS_H
#define	INC_MGUI_FORMSNAPPARMS_H

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_RVC_VECTTOOL_H
#include <rvc/vecttool.h>
#endif


class RVC::VECTORTOOLKIT::SNAPPARMS::PANEL : public MGUI::FORM_COMPOSITE {
	public:
		enum PROMPT {
			PROMPT_SnapElement = 		0x0001,
			PROMPT_SnapType = 			0x0002,
			PROMPT_SnapEnd = 				0x0004,
			PROMPT_SnapDistance = 		0x0008,
			PROMPT_SnapBackAngle = 		0x0010,
			PROMPT_RemoveOvershoots = 	0x0020,
			PROMPT_CrossLines = 			0x0040,
			PROMPT_VectorSnapDist = 	0x0080,
			PROMPT_PixelSnapDist = 		0x0100
			};

		//! Constructor.
		PANEL (
			);

		//! Destructor.
		virtual ~PANEL (
			);

		//! Create
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			PROMPT PromptSet,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedWidth
			);
			
		int GetAngleUnits (
			) const;
			
		int GetDistanceUnits (
			) const;
			
		const RVC::VECTORTOOLKIT::SNAPPARMS& GetSnapParms (
			) const;

		void SetAngleUnits (
			int AngleUnit
			);

		void SetDelegateGetPixelScale (
			DELEGATE_DOUBLE_NOPARMS delegate
			) { m_DelegateGetPixelScale = delegate; }
			
		void SetDelegateOnChanged (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChanged = delegate; }
			
		void SetDistanceUnits (
			int DistUnit
			);
			
		void SetScaleToMeters (
			double ScaleToMeters
			);
			
		void SetSnapParms (
			const RVC::VECTORTOOLKIT::SNAPPARMS& SnapParms
			);

	protected:

		//! Called to get pixel scale.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual double v_GetPixelScale ();

		//! Called when setting changed.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnChanged ();
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		enum SPECIALUNITS {
			SPECIALUNIT_Pixels = 254,
			SPECIALUNIT_ObjectUnits
			};
	
		RVC::VECTORTOOLKIT::SNAPPARMS m_SnapParms;
		double m_ScaleToMeters;
		int m_AngleViewUnit;
		int m_DistViewUnit;

		DELEGATE_DOUBLE_NOPARMS m_DelegateGetPixelScale;
		DELEGATE_VOID_NOPARMS m_DelegateOnChanged;
	
		MGUI::FORM_COMBOBOX m_SnapElement;
		MGUI::FORM_COMBOBOX m_SnapEnd;
		MGUI::FORM_COMBOBOX m_SnapType;
		MGUI::CTRL_TOGGLEBUTTON m_RemoveOvershoots;
		MGUI::CTRL_TOGGLEBUTTON m_CrossLines;
		MGUI::FORM_EDIT_NUMBER m_SnapDist;
		MGUI::CTRL_COMBOBOX m_SnapDistUnits;
		MGUI::FORM_EDIT_NUMBER m_SnapBackAngle;
		MGUI::CTRL_COMBOBOX m_SnapBackUnits;

		void OnCrossLines ();
		void OnRemoveOvershoots ();
		void OnSnapBackAngle ();
		void OnSnapBackUnits ();
		void OnSnapDist ();
		void OnSnapDistUnits ();
		void OnSnapElement ();
		void OnSnapEnd ();
		void OnSnapType ();
		
		void SetSnapCtrls ();
		void SetSnapState ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	
	};
	
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(RVC::VECTORTOOLKIT::SNAPPARMS::PANEL::PROMPT)
#endif
	
#endif		//	INC_MGUI_FORMSNAPPARMS_H
