/**
 * \file <mgui/gadgettextblock.h>
 * \brief MGUI::GADGET_TEXTBLOCK definitions
 *
 * \if NODOC
 * $Id: gadgettextblock.h_v 1.2 2011/12/30 14:28:08 mju Exp $
 *
 * $Log: gadgettextblock.h_v $
 * Revision 1.2  2011/12/30 14:28:08  mju
 * Use canvasdevice_Memory.
 *
 * Revision 1.1  2011/03/21 19:07:04  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_GADGETTEXTBLOCK_H
#define	INC_MGUI_GADGETTEXTBLOCK_H

#ifndef	INC_MGUI_GADGET_H
#include <mgui/gadget.h>
#endif

#ifndef	INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif


namespace MGUI {

//=====================================================================================================================

//! Simple textblock gadget with tooltip support.
class GADGET_TEXTBLOCK : public MGUI::GADGET {
	public:

		//! Contructor.
		GADGET_TEXTBLOCK (
			MGUI::TOOLCANVAS *pCanvas,			//!< Canvas to contain gadget
			PLACEMENT PlacementsAllowed,		//!< Allowed placements
			FLAGS flags = FLAG_None,
			CANVASDEVICES CanvasDevices = CANVASDEVICE_Memory
			);

		//! Destructor.
		~GADGET_TEXTBLOCK ();

		//! Get complete text style.
		const TEXTSTYLE& GetTextStyle () const
			{ return (m_TextBlock.GetStyle()); }

		//! Set text string.
		void SetText (
			const MISTRING& text
			) { m_TextBlock.SetText(text); }

		//! Set text background color.
		//! Will automatically set halo style based on whether transparent or not.
		void SetTextColorBG (
			const COLOR& color
			);

		//! Set text foreground color.
		void SetTextColorFG (
			const COLOR& color
			);

		//! Set text font.
		//! Will use default font if not specified.
		void SetTextFont (
			const MISTRING& FontName
			);

		//! Set text size in pixels.
		void SetTextSize (
			UINT16 TextSize
			);

		//! Set complete text style.
		void SetTextStyle (
			const TEXTSTYLE& style
			) { m_TextBlock.SetStyle(style); }

		//! Set tooltip string.
		void SetToolTip (
			const MISTRING& tooltip
			) { m_ToolTip = tooltip; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MGD::TEXTBLOCK m_TextBlock;
		MISTRING m_ToolTip;

		// GADGET overrides
		virtual MGUI::POINT v_ComputeSize ();
		virtual void v_Draw (MGD::CONTEXT *gc, MGUI::CANVAS::DEVICETYPE DeviceType);
		virtual bool v_OnToolTipRequest (MGUI::CTRL_TOOLTIP& tooltip, MGUI::POINT& ptCanvas);
		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_GADGETTEXTBLOCK_H
