/**
 * \file mgui/sash.h
 * \brief Definitions for MGUI::CTRL_SASH.
 *
 * \if NODOC
 * $Id: sash.h_v 1.9 2011/11/22 21:37:41 mju Exp $
 *
 * $Log: sash.h_v $
 * Revision 1.9  2011/11/22 21:37:41  mju
 * Fix dox dependencies.
 *
 * Revision 1.7  2010/10/25 14:36:38  mju
 * Add "hasmore" to v_onPointerMove.
 *
 * Revision 1.6  2008/02/22 22:36:36  mju
 * Add setDelegateOnMoved.
 *
 * Revision 1.1  2006/10/26 14:12:31  dwilliss
 * Initial revision
 * \endif
 **/

#ifndef	INC_MGUI_SASH_H
#define	INC_MGUI_SASH_H

#ifndef	INC_MGUI_CANVAS_H
#include <mgui/canvas.h>
#endif


namespace MGUI {

//===================================================================================================================
//! Sash control for sizing adjacent panes.
class CTRL_SASH : public MGUI::CANVAS {
	public:
		
		//! Constructor.
		CTRL_SASH ();

		//! Destructor.
		virtual ~CTRL_SASH ();

		//! Create a sash.  
		//! Note: the ParentPane must remain in scope for the life of the
		//! sash, or at least while there's a chance of user interaction with
		//! the sash.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,
			LAYOUT_ORIENTATION Orientation
			);

		//! Set delegate to call after sash moved.
		//! Useful for recording settings.
		void SetDelegateOnMoved (
			DELEGATE_VOID_NOPARMS delegate
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class PRIV;
		PRIV* m_priv;

		virtual void v_OnPaint (MGD::CONTEXT*);
		virtual void v_OnPointerButtonDown (POINT, POINTERBUTTON button, KEYSTATE);
		virtual void v_OnPointerButtonUp (POINT, POINTERBUTTON button, KEYSTATE);
		virtual void v_OnPointerMove (POINT, POINTERBUTTON button, KEYSTATE, bool);

		CTRL_SASH (const CTRL_SASH&);
		CTRL_SASH& operator= (const CTRL_SASH&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace MGUI

#endif	// INC_MGUI_SASH
