/**
 * \file mgui/spatmodimageview.h
 * \brief Definitions for MGUI::FORM_SPATMOD_IMAGE_VIEW.
 *
 * \if NODOC
 * $Id: spatmodimageview.h_v 1.8 2011/11/22 21:37:46 mju Exp $
 *
 * $Log: spatmodimageview.h_v $
 * Revision 1.8  2011/11/22 21:37:46  mju
 * Fix dox dependencies.
 *
 * Revision 1.7  2011/09/01 18:05:24  mju
 * Fix doc hdr.
 *
 * Revision 1.6  2011/09/01 18:04:53  mju
 * Exclude macro from dox.
 *
 * Revision 1.5  2007/03/27 21:56:08  bcolombe
 * Changed Delegates to take pointer to viewer
 * Added Get and SetViewNum to handle dbeditor's multiple image viewers
 *
 * Revision 1.4  2007/03/27 15:12:53  bcolombe
 * Added SetDelegateOnNextPhoto and PrevPhoto
 *
 * Revision 1.3  2006/11/16 15:04:10  dwilliss
 * added a v_Close
 *
 * Revision 1.2  2006/11/02 23:24:40  dwilliss
 * Added STYLE enum to control tool bar
 *
 * Revision 1.1  2006/10/30 21:42:52  dwilliss
 * Initial revision
 *
 * \endif
 **/

#ifndef	INC_MGUI_SPATMODIMAGEVIEW_H
#define	INC_MGUI_SPATMODIMAGEVIEW_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
namespace SPATMOD {
	namespace IMAGE {
		class STAGE;
		}
	}
namespace MGUI {
	class CANVAS;
	}
#endif

namespace MGUI {

//===================================================================================================================

//! A viewer for SPATMOD::IMAGE::STAGE
class FORM_SPATMOD_IMAGE_VIEW : public MGUI::FORM_COMPOSITE {
	public:
		
		enum STYLE {
			STYLE_Default  = 0,
			STYLE_NoToolBar = 0x00000001
			};

		//! Constructor.
		FORM_SPATMOD_IMAGE_VIEW (
			);

		//! Destructor.
		virtual ~FORM_SPATMOD_IMAGE_VIEW (
			);

		//! Create a sash.  
		//! Note: the ParentPane must remain in scope for the life of the
		//! sash, or at least while there's a chance of user interaction with
		//! the sash.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,
			int width,
			int height,
			STYLE style = STYLE_Default,
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_Expand,
			SPATMOD::IMAGE::STAGE* stage = 0,
			bool bTakeOwnership = false
			);

		ERRVALUE Display (
			);

		CANVAS& GetCanvas (
			);

		//! Set delegate to call when user presses next photo button
		void SetDelegateOnNextPhoto (
			FastDelegate<void(FORM_SPATMOD_IMAGE_VIEW *)> delegate
			);

		//! Set delegate to call when user presses previous photo button
		void SetDelegateOnPrevPhoto (
			FastDelegate<void(FORM_SPATMOD_IMAGE_VIEW *)> delegate
			);

		//! Set the image stage to display.  
		void SetStage (
			SPATMOD::IMAGE::STAGE* stage,
			bool bTakeOwnership 
			);

		//! Set the image viewer window number
		void SetViewNum (const INT32& ViewNum) {m_ViewNum = ViewNum;}

		//! Get the image viewer window number
		INT32 GetViewNum() {return m_ViewNum;}

	private:
	#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_ViewNum;

		class PRIV;
		class MYCANVAS;
		PRIV* m_priv;

		friend class MYCANVAS;

		FORM_SPATMOD_IMAGE_VIEW (const FORM_SPATMOD_IMAGE_VIEW&);
		FORM_SPATMOD_IMAGE_VIEW& operator= (const FORM_SPATMOD_IMAGE_VIEW&);
	#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MGUI::FORM_SPATMOD_IMAGE_VIEW::STYLE)
#endif

//! Dialog wrapper around MGUI::FORM_SPATMOD_IMAGE_VIEW
class DLG_SPATMOD_IMAGE_VIEW : public MGUI::DLGSHELL {
	public:
		// Constructor.
		// The optional height and width parameters are the size of the actual
		// view canvas
		DLG_SPATMOD_IMAGE_VIEW (
			INT32 width = 200,
			INT32 height = 200,
			FORM_SPATMOD_IMAGE_VIEW::STYLE style = FORM_SPATMOD_IMAGE_VIEW::STYLE_Default
			);

		virtual ~DLG_SPATMOD_IMAGE_VIEW();
		
		//! Display the image
		ERRVALUE Display (
			) {
			return (m_View.Display());
			}

		FORM_SPATMOD_IMAGE_VIEW& GetFormImageView (
			) { return (m_View); }

		void SetDelegateOnClose (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnClose = delegate; }

		//! Set the image stage to display.  
		void SetStage (
			SPATMOD::IMAGE::STAGE* stage,
			bool bTakeOwnership 
			) {
			m_View.SetStage(stage, bTakeOwnership);
			}

		//! Set the image viewer window number
		void SetViewNum (const INT32& ViewNum) {m_View.SetViewNum(ViewNum);}

		//! Get the image viewer window number
		INT32 GetViewNum() {return m_View.GetViewNum();}


	private:
		FORM_SPATMOD_IMAGE_VIEW m_View;
		INT32 m_Width, m_Height;
		FORM_SPATMOD_IMAGE_VIEW::STYLE m_Style;
		DELEGATE_VOID_NOPARMS m_DelegateOnClose;

		ERRVALUE v_CreateContent();
		void v_OnOpen();
		void v_OnClose();
	};

//===================================================================================================================
}	// End namespace MGUI

#endif	// INC_MGUI_SPATMODIMAGEVIEW_H
