/**
 * \file mgui/toolbreakline.h
 * \brief MGUI::TOOL_BREAKLINE class definition
 *
 * \if NODOC
 * $Id: toolbreakline.h_v 1.4 2011/12/09 20:22:51 mju Exp $
 *
 * $Log: toolbreakline.h_v $
 * Revision 1.4  2011/12/09 20:22:51  mju
 * No context in v_draw.
 *
 * Revision 1.3  2011/06/21 14:02:30  mju
 * *** empty log message ***
 *
 * Revision 1.1  2008/08/21 22:30:05  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_TOOLBREAKLINE_H
#define	INC_MGUI_TOOLBREAKLINE_H

#ifndef	INC_MGUI_TOOLBASE_H
#include <mgui/toolbase.h>
#endif

#ifndef	INC_MI32_POLYLINE_H
#include <mi32/polyline.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Tool to break line into multiple sections.
class TOOL_BREAKLINE : public TOOL_BASE {
	public:

		//! Constructor.
		explicit TOOL_BREAKLINE (
			TOOLCANVAS *canvas
			);

		//! Destructor.
		virtual ~TOOL_BREAKLINE ();
		
		//! Get the lines defined by the tool's break points.
		void GetLines (
			POLYLINELIST& LineList
			) const;

		//! Set the line to break up
		void SetLine (
			const POLYLINE& PolyLine
			);
			
		//! Set marker type and size.
		//! The default marker is MARKER_Plus with a size of 9 pixels.
		void SetMarker (
			MARKER marker,								//!< New marker type
			UINT16 size = 0							//!< New marker size, 0 to retain current size
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		enum EDITMODE {
			EDITMODE_Automatic,
			EDITMODE_Add,
			EDITMODE_Edit,
			EDITMODE_Delete
			};

		enum ZONE {
			ZONE_NoFunction = 0,
			ZONE_Outside,
			ZONE_BreakPoint,
			ZONE_Line
			};
			
		struct BREAKPOINT {
			DPOINT3D m_Point;
			double m_DistFromStart;
			INT32 m_LineSegNum;
			bool m_SegmentDeleted;
			
			BREAKPOINT () { memset(this, 0, sizeof(BREAKPOINT)); }
			};

		//! Tool settings
		MARKER m_marker;
		UINT16 m_markersize;
		EDITMODE m_EditMode;
		//! Tool position values
		POLYLINE m_PolyLine;
		int m_ActiveIndex;						//!< Index of active breakpoint
		MILIST<BREAKPOINT> m_BreakPointList;
		//! Values used during interaction
		ZONE m_CursorZone;						//!< Current cursor zone
		DPOINT2D m_prevpt;
		double m_TooltoWindowScale;
		bool m_NotifyChangeElem;

		//! Manual Point Entry form forward declaration
		class MANUAL_BREAKLINE;
		friend class MANUAL_BREAKLINE;		//!< Used to call the DELEGATE_ONSETPOINT delegate
		class FORM_BREAKLINE;
		FORM_BREAKLINE* m_pFormBreakLine;

		//! Methods
		void ActivateClosestPoint (const DPOINT2D&);
		void AddBreakPoint (const DPOINT2D&);
		void AdjustTool (DPOINT2D&, CHANGESEQ);
		void ChangeBreakPoint (INT32 idx, const DPOINT3D&);
		void ComputeCursorZone (const DPOINT2D&);
		void DeleteBreakPoint (int);
		void DrawNeighborBreakPoints (MGD::CONTEXT*, const LRECT2D&, int SkipPoint) const;
		void DrawLine (MGD::CONTEXT*, const POLYLINE&, const DRECT2D&) const;
		void DrawTool (MGD::CONTEXT*, const DRECT2D&, int SkipBreakPointIdx = -1) const;
		void MarkLineSegment (const DPOINT2D&);
		void SetActiveBreakPointIndex (int);
		void UpdateCursor () const;

		// TOOL_BASE overrides.
		virtual MGUI::FORM* v_CreateControlForm (LAYOUT_PANE_BASE& ParentPane, CTRLOPT ctrlopt);
		virtual MANUAL_BASE* v_CreateManualBase ();
		virtual void v_DoDraw ();
		virtual void v_DoMove (double dx, double dy, ADJUSTFLAGS flags);
		virtual void v_OnClear ();
		virtual int v_GenerateRegion (REGION2D& region) const;
		virtual bool v_OnKey (MGUI::POINT point, KEYCODE keycode, KEYSTATE keystate);
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual bool v_OnPointerButtonUp (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnSetTransToWindow ();

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_TOOLBREAKLINE_H
