/**
 * \file dispinfo.h <mi32/dispinfo.h>
 * \brief Definitions for DISPLAYINFO
 *
 * \if NODOC
 * $Id: dispinfo.h_v 1.9 2007/10/30 16:17:55 mju Exp $
 *
 * $Log: dispinfo.h_v $
 * Revision 1.9  2007/10/30 16:17:55  mju
 * Add ctor.
 *
 * Revision 1.8  2007/07/12 17:09:37  mju
 * Add getPixelColor and getColorPixel.
 *
 * Revision 1.7  2004/07/27 17:34:43  dwilliss
 * Added members for keeping track of alpha mask
 *
 * Revision 1.6  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.5  2002/05/14 20:26:37  dwilliss
 * Added a new line to the end of the file to make gcc shut up about it
 *
 * Revision 1.4  2000/06/07 13:41:10  mju
 * Include stddefns.
 *
 * Revision 1.3  2000/05/19 13:47:10  sparsons
 * Documentation.
 *
 * Revision 1.2  2000/05/18 17:05:05  sparsons
 * Documentation.
 *
 * Revision 1.1  1999/04/28 15:08:28  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_DISPINFO_H
#define	INC_MI32_DISPINFO_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

struct COLOR;

//----------------------------------------------------------------------------
//		DISPLAYINFO structure															
//----------------------------------------------------------------------------

struct DISPLAYINFO {

	UINT8 HasColorMap;						//!< Set if display has a colormap 
	UINT8 ImageMSBFirst;						//!< Set if MSB first in "image" line 
	short depth;								//!< Pixmap depth in bits 
	INT32 width;								//!< Width of screen in pixels 
	INT32 height;								//!< Height of screen in pixels 
	int NumColors;								//!< Number of colors 
	short RedBits;
	short GreenBits;
	short BlueBits;
	short RedShift;							//!< Bits to shift left if DirectColor or TrueColor 
	short GreenShift;
	short BlueShift;							//!< Indicates display is coming towards you very fast 
	UINT32 RedMask;
	UINT32 GreenMask;
	UINT32 BlueMask;
	short bits_per_pixel;					//!< Mainly for PutImage operations, can be different than depth 
	short AlphaBits;
	short AlphaShift;
	UINT32 AlphaMask;

	DISPLAYINFO ();

	MPIXEL GetColorPixel (
		const COLOR& color
		) const;

	void GetPixelColor (
		MPIXEL pixel,
		COLOR& color
		) const;
	};


#endif	//!< INC_MI32_DISPINFO_H

