/**
 * \file mi32/lidarclassifiernoise.h
 * \brief LIDAR::CLASSIFIER_NOISE definitions
 *
 * \if NODOC
 * $Id: lidarclassifiernoise.h_v 1.1 2012/09/24 20:24:37 mju Exp $
 *
 * $Log: lidarclassifiernoise.h_v $
 * Revision 1.1  2012/09/24 20:24:37  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_LIDARCLASSIFIERNOISE_H
#define	INC_MI32_LIDARCLASSIFIERNOISE_H

#ifndef	INC_MI32_LIDARCLASSIFIER_H
#include <mi32/lidarclassifier.h>
#endif

#ifndef	INC_MI32_RANGE_H
#include <mi32/range.h>
#endif

namespace LIDAR {

//=====================================================================================================================

//! Lidar classifier to detect low/high noise points.
class CLASSIFIER_NOISE : public CLASSIFIER {
	public:

		//! Constructor.
		CLASSIFIER_NOISE ();

		//! Destructor.
		~CLASSIFIER_NOISE ();

		//! Get whether to mark points as withheld.
		bool GetMarkAsWithheld () const
			{ return (m_MarkAsWithheld); }

		//! Get maximum difference above neighbors.
		double GetMaxDifferenceAbove () const
			{ return (m_MaxAbove); }

		//! Get maximum difference below neighbors.
		double GetMaxDifferenceBelow () const
			{ return (m_MaxBelow); }

		//! Get minimum number of neighbors.
		UINT16 GetMinNeighbors () const
			{ return (m_MinNeighbors); }

		//! Get overall range.
		const DOUBLE_RANGE& GetRangeOverall () const
			{ return (m_RangeOverall); }

		//! Get range for sparse points.
		const DOUBLE_RANGE& GetRangeSparse () const
			{ return (m_RangeSparse); }

		//! Get search distance.
		double GetSearchDistance () const
			{ return (m_SearchDistance); }

		//! Run classifier.
		ERRVALUE Run (
			LAS::DATASET& dataset,
			const DRECT2D& extents
			);

		//! Set whether to mark noise points as "withheld".
		void SetMarkAsWithheld (
			bool MarkAsWithheld
			) { m_MarkAsWithheld = MarkAsWithheld; }

		//! Set maximum difference above neighbors.
		void SetMaxDifferenceAbove (
			double MaxAbove
			) { m_MaxAbove = MaxAbove; }

		//! Set maximum difference below neighbors.
		void SetMaxDifferenceBelow (
			double MaxBelow
			) { m_MaxBelow = MaxBelow; }

		//! Set minimum number of neighbors.
		void SetMinNeighbors (
			UINT16 MinNeighbors
			) { m_MinNeighbors = MinNeighbors; }

		//! Set overall range allowed.
		void SetRangeOverall (
			const DOUBLE_RANGE& RangeOverall
			) { m_RangeOverall = RangeOverall; }

		//! Set range for sparse points.
		void SetRangeSparse (
			const DOUBLE_RANGE& RangeSparse
			) { m_RangeSparse = RangeSparse; }

		//! Set search distance.
		void SetSearchDistance (
			double SearchDistance
			) { m_SearchDistance = SearchDistance; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		enum OUTPUT {
			OUTPUT_LowNoise = 0,
			OUTPUT_HighNoise = 1
			};

		class ENGINE;

		DOUBLE_RANGE m_RangeOverall;
		DOUBLE_RANGE m_RangeSparse;
		double m_MaxAbove;
		double m_MaxBelow;
		double m_SearchDistance;
		UINT16 m_MinNeighbors;
		bool m_MarkAsWithheld;

		// Overrides.
		virtual double v_GetMargin () const;
		virtual UINT64 v_GetMaxPointsDesired () const;
		virtual void v_IniRead (INIHANDLE inihnd);
		virtual void v_IniWrite (INIHANDLE inihnd) const;
		virtual void v_JobGetValues (MIJOB& mijob) const;
		virtual ERRVALUE v_JobRead (const MIJOB& mijob);
		virtual void v_Reset ();
		#endif
	};

//=====================================================================================================================

}	// End namespace LIDAR

#endif	// INC_MI32_LIDARCLASSIFIERNOISE_H
