/**
 * \file mtimer.h <mi32/mtimer.h>
 * \brief Header file for MTIMER class
 *
 * \if NODOC
 * $Id: mtimer.h_v 1.15 2008/05/21 19:38:40 mju Exp $
 *
 * $Log: mtimer.h_v $
 * Revision 1.15  2008/05/21 19:38:40  mju
 * Add reportElapsedAbbr.
 *
 * Revision 1.14  2007/11/07 22:41:59  mju
 * Add reportElapsed.
 *
 * Revision 1.13  2006/10/06 14:11:17  scowan
 * *** empty log message ***
 *
 * Revision 1.12  2006/10/06 14:10:56  scowan
 * Removed unused define.
 *
 * Revision 1.11  2006/10/06 14:09:48  scowan
 * Removed legacy code that should have been removed in 7.2.
 *
 * Revision 1.10  2006/10/06 12:38:37  mju
 * Remove textid hdr, don't need for this hdr.
 *
 * Revision 1.9  2005/03/21 17:25:25  scowan
 * Removed group key version of report method.
 *
 * Revision 1.8  2005/02/16 22:29:32  mju
 * Remove mtimerReportStr as unused.
 *
 * Revision 1.7  2005/02/16 22:25:29  mju
 * Remove report variant using group/key except for DLL build.
 *
 * Revision 1.6  2004/02/16 21:01:45  mju
 * Warn on deprecated usage.
 *
 * Revision 1.5  2003/10/27 16:26:12  dwilliss
 * Fixed Mike's deprecatation.   Have to allow the prototype to exist if
 * compiling misystem
 *
 * Revision 1.4  2003/10/09 13:44:30  mju
 * Remove rest of deprecated stuff.
 *
 * Revision 1.3  2003/10/08 22:03:21  mju
 * Remove unused deprecated stuff.
 *
 * Revision 1.2  2003/10/08 17:32:00  mju
 * Fix to compile.
 *
 * Revision 1.1  2003/10/08 16:40:01  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_MTIMER_H
#define	INC_MI32_MTIMER_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

class MISTRING;

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


//=====================================================================================================================
//! Interval timer, used to time operations.
//! Timer is accurate to 100ns on Windows and 1ms on other platforms.
struct CLASSLIBEXPORT MTIMER {
	public:

		//! Default constructor
		MTIMER ();

		//! Destructor.
		~MTIMER ();

		//! Clear timer.
		//! Also stops timer.
		void Clear ();

		// Get timer value as formatted string.
		void GetString (
			MISTRING& string
			) const;

		//! Get the value of the timer in seconds.
		double GetValue (
			) const;

		//! Determine if timer is currently 'on'.
		bool IsOn (
			) const;

		//! Report "Time to ???: xxx xxx xxx".
		//!
		//! The ??? part is retrieved from the text resource file using the TEXTID provided.
		//! The rest of the message is built from text resource strings and the actual time.
		//!
		//! If TEXTID__None is specified then only the time is reported.
		void Report (
			MISTRING& string,
			TEXTID textid = TEXTID__None
			) const;

		//! Report elapsed time in simple format.
		//! 0-90 seconds reported as seconds.
		//! 1.5 to 90 minutes reported as minutes.
		//! > 90 minutes reported as hours.
		void ReportElapsed (
			MISTRING& string
			) const;

		//! Report elapsed time in simple format with abbreviated unit.
		//! 0-90 seconds reported as seconds.
		//! 1.5 to 90 minutes reported as minutes.
		//! > 90 minutes reported as hours.
		void ReportElapsedAbbr (
			MISTRING& string
			) const;

		//! Report in "00:00" format
		void ReportShort (
			MISTRING& string,
			int places = 0		//!< number of decimal places in seconds (eg, 0:00.00)
			) const;

		//! Start timer.
		void Start ();

		//! Set the value of the timer
		void Set (
			double seconds
			);

		//! Stop timer.
		//! Does not clear the timer.
		void Stop ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		double m_total;
		double m_StartTime;
		UINT8 m_TimerOn;
		UINT8 padding[7];

		CHECKSIZE(24)
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#undef CLASSLIBEXPORT
#endif	// INC_MI32_MTIMER_H
