/**
 * \file rvc/oprastertogeombound.h
 * \brief Operation to convert raster to geometric by boundary tracing
 *
 * \if NODOC
 * $Id: oprastertogeombound.h_v 1.3 2011/09/02 12:53:11 mju Exp $
 *
 * $Log: oprastertogeombound.h_v $
 * Revision 1.3  2011/09/02 12:53:11  mju
 * Dox hdr.
 *
 * Revision 1.2  2006/07/21 12:54:20  mju
 * Rename to remove converter_ prefix.
 *
 * \endif
**/

#ifndef	INC_RVC_OPRASTERTOGEOMBOUND_H
#define	INC_RVC_OPRASTERTOGEOMBOUND_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class OBJITEM;
	}
#endif


namespace RVC {
namespace OP {

//=====================================================================================================================

//! Converter for performing raster to geometric conversion by boundary tracing.
class RASTER_TO_GEOMETRIC_BOUNDARY {
	public:

		//! Default constructor.
		RASTER_TO_GEOMETRIC_BOUNDARY ();

		//! Convert source to target using current settings.
		//! The target object type will be determined from the specified target location settings.
		ERRVALUE Process (
			const RVC::OBJITEM& ObjItemSrc,				//!< Source location
			RVC::OBJITEM& ObjItemTgt						//!< Target location passed/returned
			) const;

		//! Set whether to compute "standard attributes" for result.
		//! Default is to not compute standard attributes.
		void SetComputeStdAttributes (
			bool ComputeStdAttributes
			) { m_ComputeStdAttributes = ComputeStdAttributes; }

		//! Set whether to make implied georeference for result.
		//! Default is to not use implied georeference.
		//! If implied georeference is not used, geometric coordinates will be integer values
		//! corresponding to X/Y raster positions, and a transformation to map coordinates will
		//! be saved.  Otherwise, the coordinates will be converted and save directly as map
		//! coordinates and an implied georeference will be saved.
		void SetMakeImpliedGeoref (
			bool MakeImpliedGeoref
			) { m_MakeImpliedGeoref = MakeImpliedGeoref; }

		//! Set whether to optimize resulting object for search.
		//! Default is to not optimize.
		//! Generally requires temporary workspace equal to size of resulting object and
		//! and additional pass for optimization.  Ignored if not supported for object type.
		void SetOptimizeForSearch (
			bool OptimizeForSearch
			) { m_OptimizeForSearch = OptimizeForSearch; }

		//! Set whether to save database for resulting polygons.
		//! Default is to save database.
		//! If database is saved then any database from the source raster will be copied to
		//! the target and adjusted accordingly to maintain relationships.
		//! Furthermore, if raster has a color palette a style assignment table will be automatically
		//! created to maintain the same colors.
		void SetSaveDatabase (
			bool SaveDatabase
			) { m_SaveDatabase = SaveDatabase; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		bool m_ComputeStdAttributes;
		bool m_MakeImpliedGeoref;
		bool m_OptimizeForSearch;
		bool m_SaveDatabase;
		#endif	// GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_RVC_OPRASTERTOGEOMBOUND_H
