/**
 * soauiservicetree.h - SOA ARCIMS User Interface for connection Connection
 *
 * This was origionally a private control embeded in 
 * SOA::WMS::UI_FORM_CONNECTION.  When that was duplicated for ARCIMS, it
 * seemed a bit redundant, so I split it out into its own module that they
 * can both share
 *
 * $Id: uiservicetree.h_v 1.3 2010/08/11 22:02:57 bcolombe Exp $
 *
 * $Log: uiservicetree.h_v $
 * Revision 1.3  2010/08/11 22:02:57  bcolombe
 * Added delegate for double click on service
 *
 * Revision 1.2  2007/10/26 21:31:35  dwilliss
 * More image indexii
 *
 * Revision 1.1  2007/09/28 14:39:35  dwilliss
 * Initial revision
 *
 *
 **/

#ifndef INC_SOA_UISERVICETREE_H
#define INC_SOA_UISERVICETREE_H

#ifndef	INC_SOA_UISERVERLIST_H
#include <soa/uiserverlist.h>
#endif

#ifndef	INC_MGUI_TREE_H
#include <mgui/tree.h>
#endif

//=====================================================================================================================

namespace SOA {


class CTRL_SERVICETREE : public MGUI::CTRL_TREE {
	public:

		typedef fastdelegate::FastDelegate<void(ITEMHND,const LPOINT2D&)> DELEGATE_BUTTON;
		typedef fastdelegate::FastDelegate<void(SOA::SERVERLIST::ITEM&)> DELEGATE_SERVERLISTITEM;

		enum IMAGEIDX {
			IMAGEIDX_Blank = 0,
			IMAGEIDX_Folder,
			IMAGEIDX_Catalog,
			IMAGEIDX_Layer,
			IMAGEIDX_ServiceWMS,
			IMAGEIDX_ServiceWFS,
			IMAGEIDX_ServiceArcIMS,
			};

		CTRL_SERVICETREE (
			);

		~CTRL_SERVICETREE ();

		ITEMHND AddItem (
			SERVERLIST::ITEM *pParentItem,
			const SERVERLIST::ITEM *pPreviousItem,
			const SERVERLIST::ITEM& Item,
			bool RemoveExisting,
			IMAGEIDX *img=0
			);

		const SOA::SERVERLIST& GetServerList (
			) const { return (m_ServerList); }

		SOA::SERVERLIST& GetServerList (
			) { return (m_ServerList); }

		const FILEPATH& GetServerListPath (
			) const { return (m_PathServerList); }

		ERRVALUE LoadServerList (
			const char *filename
			);

		 void ClearServerList (
			);

		void OnNotifyItemAdded (
			const SERVERLIST::ITEM *pParentItem,
			const SERVERLIST::ITEM *pItemAdded
			);

		bool RemoveItem (
			ITEMHND hndItem
			);

		ERRVALUE SaveServerList (
			);

		void SetDelegateOnRightButtonDown (
			DELEGATE_BUTTON delegate
			) { m_DelegateOnRightButtonDown = delegate; }

		void SetDelegateOnSelectService (
			DELEGATE_SERVERLISTITEM delegate
			) { m_DelegateOnSelectService = delegate; }

		void SetDelegateOnDoubleClickService (
			DELEGATE_SERVERLISTITEM delegate
			) { m_DelegateOnDoubleClickService = delegate; }

		void SetLayerSelected(const SOA::SERVERLIST::ITEM& item);

#ifndef GENERATING_DOXYGEN_OUTPUT
	private:

		UINT64 m_LastClickTime;
		FILEPATH m_PathServerList;
		SOA::SERVERLIST m_ServerList;
		DELEGATE_BUTTON m_DelegateOnRightButtonDown;
		DELEGATE_SERVERLISTITEM m_DelegateOnSelectService;
		DELEGATE_SERVERLISTITEM m_DelegateOnDoubleClickService;

		ITEMHND AddItem (ITEMHND hndParent, ITEMHND hndInsertAfter, const SOA::SERVERLIST::ITEM& item, IMAGEIDX *img=0);
		void AddServerList (ITEMHND hndParent, const SOA::SERVERLIST::ITEMLIST& ItemList);

		// MGUI::CTRL_TREE overrides.
		virtual bool v_OnChildVisibilityChanging (ITEMHND hndItem, CHILDVISIBILITY visibility);
		virtual void v_OnLeftButtonDown (ITEMHND hndItem, HITRESULT HitResult, const LPOINT2D& point, MGUI::KEYSTATE keystate);
		virtual void v_OnRightButtonDown (ITEMHND hndItem, HITRESULT HitResult, const LPOINT2D& point, MGUI::KEYSTATE keystate);
		virtual bool v_OnToolTipRequest (ITEMHND hndItem, HITRESULT HitResult, const LPOINT2D& point, MGUI::CTRL_TOOLTIP& ToolTipCtrl);

		CTRL_SERVICETREE (const CTRL_SERVICETREE&);
		CTRL_SERVICETREE& operator= (const CTRL_SERVICETREE&);
#endif
	};
	
}	// End namespace SOA

#endif // INC_SOA_UISERVICETREE_H
