/**
 * \file soa/wmsuiconnection.h
 * \brief SOA WMS User Interface for client Connection
 *
 * \if NODOC
 * $Id: wmsuiconnection.h_v 1.9 2010/11/04 17:35:00 bcolombe Exp $
 *
 * $Log: wmsuiconnection.h_v $
 * Revision 1.9  2010/11/04 17:35:00  bcolombe
 * *** empty log message ***
 *
 * Revision 1.8  2010/08/15 23:46:10  bcolombe
 * Added CreateSearchForm
 *
 * Revision 1.7  2010/08/11 21:56:02  bcolombe
 * Changes to make connection a form
 *
 * Revision 1.6  2007/08/21 20:35:03  honerkamp
 * Add search option to the wms server selection
 *
 * Revision 1.5  2007/07/31 17:48:03  honerkamp
 * Forward declare gregroup
 *
 * Revision 1.4  2007/07/31 17:41:57  honerkamp
 * Added a group param to teh dlggetconnection
 *
 * Revision 1.3  2007/02/02 22:25:23  mju
 * *** empty log message ***
 *
 * Revision 1.2  2007/01/25 22:12:18  mju
 * Add setConnection.
 *
 * Revision 1.1  2007/01/19 18:40:55  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SOA_WMSUICONNECTION_H
#define	INC_SOA_WMSUICONNECTION_H

#ifndef	INC_MGUI_FORM_H
	#include <mgui/form.h>
#endif

#ifndef INC_SOA_UISERVICETREE_H
	#include <soa/uiservicetree.h>
#endif

class MISTRINGLIST;

namespace GRE{
class GROUP;
}

namespace SOA {
namespace WMS {

class CONNECTION;

//==================================================================================================================

class UI_FORM_CONNECTION_SMALL : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		UI_FORM_CONNECTION_SMALL ();

		//! Destructor.
		virtual ~UI_FORM_CONNECTION_SMALL ();

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Get current connection.
		const WMS::CONNECTION& GetConnection (
			) const;

		//! Set delegate to call when connection changed.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set connection.
		ERRVALUE SetConnection (
			const WMS::CONNECTION& Connection,
			bool notify
			);

		//! Set server URL.
		ERRVALUE SetServerURL (
			const MISTRING& ServerURL,
			bool notify
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		UI_FORM_CONNECTION_SMALL (const UI_FORM_CONNECTION_SMALL&);
		UI_FORM_CONNECTION_SMALL& operator= (const UI_FORM_CONNECTION_SMALL&);
		#endif
	};

class UI_FORM_CONNECTION : public MGUI::FORM_COMPOSITE {
	public:

		UI_FORM_CONNECTION (
			bool bShowPreview = false,
			const GRE::GROUP* group=0
			);

		UI_FORM_CONNECTION (
			const WMS::CONNECTION& connection,
			const GRE::GROUP* group=0
			);

		~UI_FORM_CONNECTION ();

		//! Create form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Create only the search form.
		ERRVALUE CreateSearchForm (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Get current connection.
		const WMS::CONNECTION& GetConnection (
			) const;

		//! Get default layers
		const MISTRINGLIST GetDefaultLayers (
			) const;

		//! Has item selected
		bool HasConnection (
			) const;

		void OnOpen();

		void OnOK();
		
		//! Set connection.
		void SetConnection (
			const WMS::CONNECTION& connection
			);

		//! delegate when user clicks on item
		void SetDelegateOnSelectService (
			SOA::CTRL_SERVICETREE::DELEGATE_SERVERLISTITEM delegate
			);

		//! delegate when user double clicks on item
		void SetDelegateOnDoubleClickService (
			SOA::CTRL_SERVICETREE::DELEGATE_SERVERLISTITEM delegate
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		UI_FORM_CONNECTION (const UI_FORM_CONNECTION&);
		UI_FORM_CONNECTION& operator= (const UI_FORM_CONNECTION&);
		#endif
	};

//=====================================================================================================================

//! Dialog to prompt user to choose WMS.
//! @return TRUE if connection changed, FALSE if not, < 0 if error or Cancel
int DlgGetConnection (
	MDLGPARENT dlgparent,							//!< Parent for dialog
	WMS::CONNECTION& connection,					//!< Connection passed/returned
	const GRE::GROUP *group=0,						//!< Group to Match the bounding box to when searching for wms layers from catalog
	MISTRINGLIST *defaultlayers=0					//!< Layer name that the user selected in the dialog
	);

//=====================================================================================================================

}	// End namespace WMS
}	// End namespace SOA

#endif	// INC_SOA_WMSUICONNECTION_H
